// vim: ts=4:sw=4
package de.steg0.bi.jr;

import java.util.HashMap;
import java.util.Map;

import net.sf.jasperreports.engine.JRCommonText;
import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.fill.JRTextMeasurer;
import net.sf.jasperreports.engine.fill.JRTextMeasurerFactory;

/**
 * Factory for {@link JRiTextMeasurer} instances.
 * This class should be initalized (using {@link #setExporterParameters(java.util.Map)})
 * to use the same set of parameters the actual exporters use, regarding
 * font map and line break policy.
 */
public class JRiTextMeasurerFactory implements JRTextMeasurerFactory
{
    private static JRPdfExporterUtil pdfExporterUtil;
    private static Map exporterParameters=new HashMap();
    private static Float lineSpacingFactor;

    public static synchronized void setExporterParameters(Map exporterParameters)
    {
        JRiTextMeasurerFactory.exporterParameters=new HashMap(exporterParameters);
        if(JRiTextMeasurerFactory.exporterParameters==null)
        {
            JRiTextMeasurerFactory.exporterParameters=new HashMap();
        }
        JRiTextMeasurerFactory.pdfExporterUtil=null;
    }

	public static synchronized void setLineSpacingFactor(Float lineSpacingFactor)
	{
		JRiTextMeasurerFactory.lineSpacingFactor = lineSpacingFactor;
	}

    protected static synchronized JRPdfExporterUtil getExporterUtil()
    throws JRException
    {
        if(JRiTextMeasurerFactory.pdfExporterUtil==null)
        {
            JRiTextMeasurerFactory.pdfExporterUtil=new JRPdfExporterUtil(
                    JRiTextMeasurerFactory.exporterParameters);
        }
        return JRiTextMeasurerFactory.pdfExporterUtil;
    }

	public synchronized JRTextMeasurer createMeasurer(JRCommonText text)
    {
        try
        {
            return new JRiTextMeasurer(text,
                    JRiTextMeasurerFactory.getExporterUtil(),
                    JRiTextMeasurerFactory.lineSpacingFactor);
        }
        catch(JRException e0)
        {
            throw new InternalError("Creating internal data structures "+
                    "failed: "+e0);
        }
    }
}
